import React, { useEffect, useState } from 'react';
import TopNavbar from './components/TopNavbar';
import { BrowserRouter as Router, Switch, Route } from 'react-router-dom';
import Login from './components/Login';
import Objects from './components/Objects';
import PrivateRoute from './components/PrivateRoute';
import UnAuthPage from './components/UnAuthPage';
import AboutServer from './components/about_server/AboutServer';
import ErrorPage from './components/ErrorPage';
import Dashboard from './components/Dashboard';
import { observer } from 'mobx-react-lite';
import UserInfo from './components/UserInfo';
import Users from './components/Users';
import LeftMenu from './components/LeftMenu';
import ServerLogs from './components/ServerLogs';
import ServerLog from './components/ServerLog';
import FavServers from './components/FavServers';
import PermsTest from './components/PermsTest';
import Categories from './components/Categories';
import ObjectTypes from './components/ObjectTypes';
import Calls from './components/Calls';
import PageEdit from './components/PageEdit';
import { Button, Classes, Dialog, FocusStyleManager } from '@blueprintjs/core';
import { useStores } from './hooks/useStores';
import { HelmetProvider } from 'react-helmet-async';
import ObjectView from './components/Object';
import Footer from './components/Footer';
import Reset from './components/Reset';
import ApiTest from './components/ApiTest';

import ExperimentOne from './components/Experiments/ExperimentOne';
import AdminDB from './components/AdminDB';
import AutoLogin from './components/AutoLogin';
import ApiReference from './components/ApiReference';
import Relations from './components/Relations';
import ApiLog from './components/ApiLog';
import Markdown from './components/markdown/Markdown';

const App = observer(() => {
    FocusStyleManager.onlyShowFocusOnTabs();

    const { rootStore } = useStores();
    let { isLoggedIn, tryLogin, loading, lastError, clearLastError, session } = rootStore.loginStore;
    let { darkTheme } = rootStore.localStore;

    let [loginChecked, setLoginChecked] = useState(undefined);
    // Wrapped in useCallback otherwise it would be recreated each time
    // this component rerenders, hence triggering useEffect below
    let changeTheme = () => {
        if (darkTheme) {
            document.body.className = 'bp3-dark';
        } else {
            document.body.className = 'bp3-body';
        }
    };

    useEffect(() => {
        changeTheme();
    }, [darkTheme]);

    useEffect(() => {
        tryLogin(res => {
            setLoginChecked(res);
        });
    }, []);

    const containerStyle = {
        display: 'flex',
        flexDirection: 'column',
        minHeight: '100vh',
    };

    const wrapperStyle = {
        padding: 20,
        width: '100%',
        display: 'flex',
        flexDirection: 'column',
    };

    const menuStyle = {
        display: 'flex',
        height: '100%',
        flexGrow: '1',
    };

    const withMenu = Component => props => {
        return (
            <div style={menuStyle} className={'main-wrapper'}>
                <LeftMenu darkTheme={localStorage.getItem('theme') === 'dark' ? true : false} />
                <div style={wrapperStyle}>
                    <Component {...props} />
                </div>
            </div>
        );
    };

    if (!!!process.env.REACT_APP_API) {
        return (
            <div>
                <p>API is not set!</p>
            </div>
        );
    }

    if (loginChecked === undefined) {
        return (
            <div>
                <p>Initialising User...</p>
            </div>
        );
    }

    let error = (
        <Dialog
            icon="error"
            onClose={() => clearLastError()}
            title="API call error"
            isOpen={!!lastError}>
            <div className={Classes.DIALOG_BODY}>
                <p>Error: {lastError}</p>
            </div>

            <div className={Classes.DIALOG_FOOTER}>
                <div className={Classes.DIALOG_FOOTER_ACTIONS}>
                    <Button onClick={() => clearLastError()}>Close</Button>
                </div>
            </div>
        </Dialog>
    );

    return (
        <div className="App">
            <HelmetProvider>
                <Router basename={process.env.PUBLIC_URL}>
                    <div style={containerStyle}>
                        {error}
                        <TopNavbar />
                        <Switch>
                            <Route exact path="/">
                                {!!isLoggedIn ? withMenu(Dashboard) : loginChecked === false ? <UnAuthPage /> : null}
                            </Route>
                            <Route path="/login" component={Login} />
                            <Route path="/clogin" component={() => <Login customServer={true} />} />
                            <Route path="/about">
                                <p>about page</p>
                            </Route>
                            <PrivateRoute exact path="/objects" component={withMenu(Objects)} />
                            <PrivateRoute exact path="/objects/:catID" component={withMenu(Objects)} />
                            <PrivateRoute exact path="/objects/:catID/:objID" component={withMenu(ObjectView)} />
                            <PrivateRoute path="/info" component={withMenu(AboutServer)} />
                            <PrivateRoute path="/profile" component={withMenu(UserInfo)} />
                            <PrivateRoute path="/users" component={withMenu(Users)} />
                            <PrivateRoute exact path="/logs" component={withMenu(ServerLogs)} />
                            <PrivateRoute exact path="/apilog" component={withMenu(ApiLog)} />
                            <PrivateRoute path="/servers" component={withMenu(FavServers)} />
                            <PrivateRoute path="/logs/:filename" component={withMenu(ServerLog)} />
                            <PrivateRoute path="/permstest" component={withMenu(PermsTest)} permission={'modDynext'} />
                            <PrivateRoute
                                path="/permstest2"
                                component={withMenu(PermsTest)}
                                permission={'NO_SUCH_PERM'}
                            />
                            <PrivateRoute path="/apitest" component={withMenu(ApiTest)} />
                            <PrivateRoute path="/categories" component={withMenu(Categories)} />
                            <PrivateRoute path="/types" component={withMenu(ObjectTypes)} />
                            <PrivateRoute path="/relations" component={withMenu(Relations)} />
                            <PrivateRoute path="/calls" component={withMenu(Calls)} />
                            <PrivateRoute path="/pages" component={withMenu(PageEdit)} />

                            <PrivateRoute exact path="/experiments/one/:fileID" component={withMenu(ExperimentOne)} />
                            <PrivateRoute path="/admin/db" component={withMenu(AdminDB)} />
                            <PrivateRoute exact path="/apiref" component={withMenu(ApiReference)} />
                            <PrivateRoute path="/apiref/:apiSearch" component={withMenu(ApiReference)} />

                            <PrivateRoute path="/md" component={withMenu(Markdown)} />

                            <Route path="/reset" component={Reset} />
                            <Route exact path="/auth" component={AutoLogin} />
                            <Route path="*" component={ErrorPage} />
                        </Switch>
                        <Footer />
                    </div>
                </Router>
            </HelmetProvider>
        </div>
    );
});

export default App;
