import React, { useEffect, useState } from 'react';
import { observer } from 'mobx-react-lite';import { useStores } from '../hooks/useStores';
// prettier-ignore
import { Button, Switch, Collapse, ControlGroup, H1, H3, H4, HTMLTable, Icon, Intent, Tab, Tabs, Tag, Tooltip, TextArea } from '@blueprintjs/core';
import { Link, useParams } from 'react-router-dom';
import { ControlledEditor as Editor } from '@monaco-editor/react';
import { toJS } from 'mobx';
import Box from './_common/Box';
import ReactJson from 'react-json-view';

const ApiReference = observer(props => {
    const { rootStore } = useStores();
    const { apis, getAPIs, getAPIRequest, apiAuthRequest } = rootStore.loginStore;
    let { apiSearch } = useParams();
    let { darkTheme } = rootStore.localStore;

    let [collapsedSections, setCollapsedSections] = useState([]);
    let [testResult, setTestResult] = useState('');
    let [currentRequest, setCurrentRequest] = useState('');
    let [currentApi, setCurrentApi] = useState(null);
    let [currentTime, setCurrentTime] = useState(0);
    let [working, setWorking] = useState(false);
    let [treeReq, setTreeReq] = useState(true);
    let [treeRes, setTreeRes] = useState(true);

    useEffect(() => {
        getAPIs(() => {
            if (!!apiSearch) {
                getAPIRequest(apiSearch, (currentApi, currentAsm) => {
                    setCollapsedSections(c => [...c, currentAsm.AssemblyName]);
                    setCurrentApi(currentApi);
                });
            }
        });
    }, [getAPIs, apiSearch, getAPIRequest]);

    useEffect(() => {
        if (!!currentApi) {
            setCurrentRequest(toJS(currentApi.ApiRequestInJson));
            setCurrentTime(0);
            setWorking(false);
        }
    }, [currentApi]);

    function handleTest() {
        if (!!currentApi) {
            setWorking(true);
            let start = Date.now();
            apiAuthRequest(JSON.parse(currentRequest), currentApi.ApiName, res => {
                setTestResult(JSON.stringify(res));
                let end = Date.now();
                setWorking(false);
                setCurrentTime(end - start);
            });
        }
    }

    const sectionStyle = {
        color: '#5C7080',
        fontSize: '1em',
        cursor: 'pointer',
    };

    const listStyle = {
        listStyleType: 'none',
        padding: 0,
        lineHeight: 1.4,
    };

    const liStyle = {
        marginBottom: 4,
    };

    function getFancyPrivitiveType(str) {
        if (str === 'Boolean') return 'bool';
        if (str === 'Int32') return 'int';
        if (str === 'String') return 'string';
        return str;
    }

    function getFancyType(str) {
        if (str === '') return '""';

        if (str === 'System.Boolean') return 'bool';
        if (str === 'System.Int32') return 'int';
        if (str === 'System.String') return 'string';

        if (str.includes('List`1')) {
            let matches = str.match(/\[(.*?)\]/);

            if (matches) {
                let string = matches[1];
                let ret = string.split('.');
                return `List<${getFancyPrivitiveType(ret.pop())}>`;
            }
        }

        if (!str.includes('`')) {
            /*let matches = str.match(/(?<=\.)[^.]*$/);

            if (matches) {
                return matches[0];
            }*/
            return str;
            //fuck safari
        }

        return str;
    }

    const infoPanel = (
        <>
            {!!apiSearch && !!apis && !!currentApi && !!currentApi.Request && (
                <>
                    <Box>
                        <H4>
                            <Icon
                                icon="arrow-right"
                                intent={Intent.PRIMARY}
                                iconSize={18}
                                style={{ marginRight: '.2em' }}
                            />
                            <code>{currentApi.Request.DtoName}</code>
                        </H4>

                        <HTMLTable small>
                            <thead>
                                <tr>
                                    <th>Name</th>
                                    <th>Type</th>
                                    <th>Default value</th>
                                </tr>
                            </thead>
                            <tbody>
                                {currentApi.Request.Fields.map(r => {
                                    return (
                                        <tr>
                                            <td style={{ minWidth: '350px' }}>
                                                <code>{r.Name}</code>
                                            </td>
                                            <td style={{ minWidth: '100px' }}>
                                                <Tooltip content={r.Type}>
                                                    <Tag minimal>{getFancyType(r.Type)}</Tag>
                                                </Tooltip>
                                            </td>
                                            <td style={{ minWidth: '100px' }}>
                                                <Tooltip content={r.DefaultValue}>
                                                    <Tag minimal>{getFancyType(r.DefaultValue)}</Tag>
                                                </Tooltip>
                                            </td>
                                        </tr>
                                    );
                                })}
                            </tbody>
                        </HTMLTable>
                    </Box>
                    <Box>
                        <H4>
                            <Icon
                                icon="arrow-left"
                                intent={Intent.PRIMARY}
                                iconSize={18}
                                style={{ marginRight: '.2em' }}
                            />
                            <code>{currentApi.Response.DtoName}</code>
                        </H4>

                        <HTMLTable small>
                            <thead>
                                <tr>
                                    <th>Name</th>
                                    <th>Type</th>
                                    <th>Default value</th>
                                </tr>
                            </thead>
                            <tbody>
                                {currentApi.Response.Fields.map(r => {
                                    return (
                                        <tr>
                                            <td style={{ minWidth: '350px' }}>
                                                <code>{r.Name}</code>
                                            </td>
                                            <td style={{ minWidth: '100px' }}>
                                                <Tooltip content={r.Type}>
                                                    <Tag minimal>{getFancyType(r.Type)}</Tag>
                                                </Tooltip>
                                            </td>
                                            <td style={{ minWidth: '100px' }}>
                                                <Tooltip content={r.DefaultValue}>
                                                    <Tag minimal>{getFancyType(r.DefaultValue)}</Tag>
                                                </Tooltip>
                                            </td>
                                        </tr>
                                    );
                                })}
                            </tbody>
                        </HTMLTable>
                    </Box>
                </>
            )}
        </>
    );

    function tryParse(json) {
        try {
            return JSON.parse(json);
        } catch {
            return '';
        }
    }

    const editorTheme = darkTheme ? 'codeschool' : 'rjv-default';

    const testPanel = (
        <>
            <div style={{ display: 'flex' }}>
                <div style={{ minWidth: '50%', width: '50%', marginRight: '.2rem' }}>
                    <Switch checked={treeReq} label="Tree view" onChange={() => setTreeReq(!treeReq)} />
                    {!treeReq && (
                        <TextArea
                            style={{ height: '500px' }}
                            fill={true}
                            value={currentRequest}
                            onChange={e => {
                                setCurrentRequest(e.target.value);
                            }}
                        />
                    )}
                    {treeReq && !!currentRequest && (
                        <ReactJson
                            theme={editorTheme}
                            src={tryParse(currentRequest)}
                            onEdit={e => {
                                setCurrentRequest(JSON.stringify(e.updated_src));
                            }}
                            displayDataTypes={false}
                            displayObjectSize={false}
                            onAdd={() => {
                                return true;
                            }}
                            style={{
                                border: `1px solid ${darkTheme ? '#262626' : '#BFCCD6'}`,
                                borderRadius: '3px 0 0 3px',
                                height: '500px',
                                maxHeight: '500px',
                                overflow: 'auto',
                                padding: '10px',
                            }}
                        />
                    )}
                </div>
                <div style={{ minWidth: '50%' }}>
                    <Switch checked={treeRes} label="Tree view" onChange={() => setTreeRes(!treeRes)} />
                    {!treeRes && <TextArea fill={true} style={{ height: '500px' }} value={testResult} />}
                    {treeRes && !!testResult && (
                        <ReactJson
                            theme={editorTheme}
                            src={tryParse(testResult)}
                            displayDataTypes={false}
                            displayObjectSize={false}
                            style={{
                                height: '500px',
                                border: `1px solid ${darkTheme ? '#262626' : '#BFCCD6'}`,
                                borderRadius: '0 3px 3px 0',
                                maxHeight: '500px',
                                overflow: 'auto',
                                padding: '10px',
                            }}
                        />
                    )}
                </div>
            </div>
            <Box>
                <div style={{ display: 'flex', justifyContent: 'space-between' }}>
                    <div>
                        <ControlGroup vertical={false}>
                            <Button icon="reset">Reset</Button>
                            <Button loading={working} icon="send-message" intent={Intent.PRIMARY} onClick={handleTest}>
                                Go
                            </Button>
                        </ControlGroup>
                    </div>
                    <div className="bp3-running-text .modifier">
                        {!!currentTime && (
                            <p>
                                <Icon style={{ marginRight: '.2rem' }} icon={'time'} />
                                {`${currentTime} ms`}
                            </p>
                        )}
                    </div>
                </div>
            </Box>
        </>
    );

    return (
        <>
            <H1>API Reference</H1>

            {!!apis && (
                <>
                    <p>
                        {apis.Assemblies.map(a => a.Apis).flat(1).length} APIs in {apis.Assemblies.length} assemblies
                    </p>
                    <div style={{ display: 'flex' }}>
                        <div style={{ minWidth: 'fit-content', marginRight: 10 }}>
                            {apis.Assemblies.map(a => {
                                return (
                                    <>
                                        <H3
                                            style={sectionStyle}
                                            onClick={e => {
                                                if (collapsedSections.indexOf(a.AssemblyName) >= 0) {
                                                    setCollapsedSections(
                                                        collapsedSections.filter(x => x !== a.AssemblyName)
                                                    );
                                                } else {
                                                    setCollapsedSections([...collapsedSections, a.AssemblyName]);
                                                }
                                            }}>
                                            {a.AssemblyName}
                                            <Tag minimal style={{ marginLeft: '.3em' }}>
                                                {a.Apis.length}
                                            </Tag>
                                        </H3>
                                        <Collapse isOpen={collapsedSections.indexOf(a.AssemblyName) !== -1}>
                                            <ul style={listStyle}>
                                                {a.Apis.map(api => {
                                                    return (
                                                        <>
                                                            <li style={liStyle}>
                                                                {apiSearch === api.ApiName ? (
                                                                    <span>
                                                                        <strong>{api.ApiName}</strong>
                                                                    </span>
                                                                ) : (
                                                                    <Link to={`/apiref/${api.ApiName}`}>
                                                                        {api.ApiName}
                                                                    </Link>
                                                                )}

                                                                {api.IsPublic && (
                                                                    <Tooltip content="Public API">
                                                                        <Icon
                                                                            icon="eye-open"
                                                                            color="#BFCCD6"
                                                                            style={{ marginLeft: '.3em' }}
                                                                        />
                                                                    </Tooltip>
                                                                )}
                                                            </li>
                                                        </>
                                                    );
                                                })}
                                            </ul>
                                        </Collapse>
                                    </>
                                );
                            })}
                        </div>
                        <div style={{ width: '100%' }}>
                            {!!apiSearch && !!apis && !!currentApi && (
                                <>
                                    <H3 style={{ display: 'inline-block', marginRight: '.5em' }}>
                                        <code>{currentApi.ApiName}</code>
                                    </H3>
                                    <div style={{ display: 'inline-block', opacity: '50%' }}>
                                        <Icon
                                            icon="endorsed"
                                            intent={Intent.NONE}
                                            iconSize={15}
                                            style={{ marginRight: '.2em' }}
                                        />
                                        <code>{currentApi.PermisionToRun}</code>
                                    </div>
                                    <Tabs id="TabsAPI">
                                        <Tab id="info" title="Info" panel={infoPanel} />
                                        <Tab id="test" title="Test" panel={testPanel} />
                                    </Tabs>
                                </>
                            )}
                        </div>
                    </div>
                </>
            )}
        </>
    );
});

export default ApiReference;
