import { Button, Classes, Dialog, FormGroup, HTMLTable, InputGroup, Intent, Toaster } from '@blueprintjs/core';
import { FieldArray, Formik } from 'formik';
import { observer } from 'mobx-react-lite';
import React, { useEffect } from 'react';
import * as yup from 'yup';
import { useStores } from '../hooks/useStores';
import CategoryFieldTypeSelect from './CategoryFieldTypeSelect';
import Box from './_common/Box';

const CategoryEditDialog = observer(({ category, isOpen, onClose, create }) => {
    const { rootStore } = useStores();

    useEffect(() => {
        if (category && category.ID_CAT) {
            rootStore.objectsStore.getCategoryColumns(category.ID_CAT);
        }
    }, [category, rootStore.objectsStore]);

    const fieldValidationSchema = yup.object().shape({
        NAME: yup.string().required(),
        DBCOL_NAME: yup.string().required(),
        DBCOL_TYPE: yup.string().required(),
        DBCOL_SIZE: yup.number().required().typeError('Must be a number'),
        // TODO: only if string
        //.positive().moreThan(0),
    });

    const validationSchema = yup.object({
        shortName: yup.string().required(),
        fullName: yup.string().required(),
        tableName: yup.string().required(),
        categoryDataColumns: yup.array().of(fieldValidationSchema),
    });

    const getTitle = c => {
        let ret;
        if (create) {
            ret = 'Создать категорию';
        } else {
            ret = 'Редактировать категорию';
        }
        return ret;
    };

    const getButtonTitle = () => {
        let ret;
        if (create) {
            ret = 'Создать';
        } else {
            ret = 'Сохранить';
        }
        return ret;
    };

    const showMsg = (msg) => {
        window.alert(msg);
    }

    let boundArrayHelpers;

    const bindArrayHelpers = arrayHelpers => {
        boundArrayHelpers = arrayHelpers;
    };

    const emptyField = {
        NAME: '',
        DESCRIPTION: '',
        DBCOL_SIZE: 0,
        DBCOL_TYPE: 'String',
    };

    return (
        <Dialog isOpen={isOpen} onClose={onClose} title={getTitle(category)} style={{ width: '80%' }}>
            <Formik
                enableReinitialize
                initialValues={{
                    shortName: category ? category.NAME_SHORT : '',
                    fullName: category ? category.NAME : '',
                    tableName: category ? category.DB_TBL_NAME : '',
                    categoryDataColumns: category ? category.CategoryDataColumns : [],
                    create: create,
                }}
                validationSchema={validationSchema}
                onSubmit={(values, actions) => {
                    rootStore.objectsStore.setCategory(category, values);
                    actions.setSubmitting(false);
                    onClose();
                }}>
                {({ handleSubmit, values, errors, setFieldValue, isSubmitting, isValid, dirty, touched }) => (
                    <>
                        <div className={Classes.DIALOG_BODY}>
                            <FormGroup label="Короткое имя" labelFor="deepcat-short-name">
                                <InputGroup
                                    id="deepcat-short-name"
                                    value={values.shortName}
                                    onChange={e => setFieldValue('shortName', e.target.value)}
                                    intent={errors.shortName && touched.shortName ? Intent.DANGER : Intent.NONE}
                                />
                            </FormGroup>

                            <FormGroup label="Имя категории" labelFor="deepcat-full-name">
                                <InputGroup
                                    id="deepcat-full-name"
                                    value={values.fullName}
                                    onChange={e => setFieldValue('fullName', e.target.value)}
                                    intent={errors.fullName && touched.fullName ? Intent.DANGER : Intent.NONE}
                                />
                            </FormGroup>

                            <FormGroup label="Имя таблицы" labelFor="deepcat-table-name">
                                <InputGroup
                                    id="deepcat-table-name"
                                    value={values.tableName}
                                    rightElement={
                                    <Button onClick={e => { rootStore.objectsStore.checkTable(values.tableName, showMsg); } }>
                                        Проверить таблицу
                                    </Button>}
                                    onChange={e => setFieldValue('tableName', e.target.value)}
                                    intent={errors.tableName && touched.tableName ? Intent.DANGER : Intent.NONE}
                                />
                            </FormGroup>

                            <HTMLTable>
                                <thead>
                                    <tr>
                                        <th>Наименование</th>
                                        <th>Описание</th>
                                        <th>Поле в БД</th>
                                        <th>Тип данных в БД</th>
                                        <th>Размер данных в БД</th>
                                    </tr>
                                </thead>
                                <tbody>
                                    { (
                                        <FieldArray
                                            name="categoryDataColumns"
                                            render={arrayHelpers => {
                                                bindArrayHelpers(arrayHelpers);
                                                return (
                                                    <>
                                                        {values.categoryDataColumns &&
                                                            values.categoryDataColumns.map((c, index) => (
                                                                <>
                                                                    <tr key={index}>
                                                                        <td key={index}>
                                                                            <InputGroup
                                                                                value={
                                                                                    values.categoryDataColumns[index]
                                                                                        .NAME
                                                                                }
                                                                                onChange={e => {
                                                                                    setFieldValue(
                                                                                        `categoryDataColumns[${index}].NAME`,
                                                                                        e.target.value
                                                                                    );
                                                                                }}
                                                                            />
                                                                        </td>
                                                                        <td>
                                                                            <InputGroup
                                                                                value={
                                                                                    values.categoryDataColumns[index]
                                                                                        .DESCRIPTION
                                                                                }
                                                                                onChange={e => {
                                                                                    setFieldValue(
                                                                                        `categoryDataColumns[${index}].DESCRIPTION`,
                                                                                        e.target.value
                                                                                    );
                                                                                }}
                                                                            />
                                                                        </td>
                                                                        <td>
                                                                            <InputGroup
                                                                                value={
                                                                                    values.categoryDataColumns[index]
                                                                                        .DBCOL_NAME
                                                                                }
                                                                                onChange={e => {
                                                                                    setFieldValue(
                                                                                        `categoryDataColumns[${index}].DBCOL_NAME`,
                                                                                        e.target.value
                                                                                    );
                                                                                }}
                                                                            />
                                                                        </td>
                                                                        <td>
                                                                            <CategoryFieldTypeSelect
                                                                                category={c}
                                                                                change={e =>
                                                                                    setFieldValue(
                                                                                        `categoryDataColumns[${index}].DBCOL_TYPE`,
                                                                                        e
                                                                                    )
                                                                                }
                                                                            />
                                                                        </td>
                                                                        <td>
                                                                            <InputGroup
                                                                                disabled={
                                                                                    values.categoryDataColumns[index]
                                                                                        .DBCOL_TYPE === 'StringNotLimit'
                                                                                }
                                                                                value={
                                                                                    values.categoryDataColumns[index]
                                                                                        .DBCOL_SIZE
                                                                                }
                                                                                onChange={e => {
                                                                                    setFieldValue(
                                                                                        `categoryDataColumns[${index}].DBCOL_SIZE`,
                                                                                        e.target.value
                                                                                    );
                                                                                }}
                                                                                intent={
                                                                                    errors.categoryDataColumns &&
                                                                                    errors.categoryDataColumns[index] &&
                                                                                    errors.categoryDataColumns[index]
                                                                                        .DBCOL_SIZE &&
                                                                                    touched.categoryDataColumns[index]
                                                                                        ? Intent.DANGER
                                                                                        : Intent.NONE
                                                                                }
                                                                            />
                                                                        </td>
                                                                        <td>
                                                                            <Button
                                                                                icon={'remove'}
                                                                                onClick={ () => 
                                                                                    {
                                                                                        if (values.categoryDataColumns[index].ID_CAT && values.categoryDataColumns[index].ID_CAT_DCOL)
                                                                                        {
                                                                                            rootStore.objectsStore.delCategoryColumn(values.categoryDataColumns[index].ID_CAT, values.categoryDataColumns[index].ID_CAT_DCOL);
                                                                                        }
                                                                                        arrayHelpers.remove(index);
                                                                                    }
                                                                                }></Button>
                                                                        </td>
                                                                    </tr>
                                                                </>
                                                            ))}
                                                    </>
                                                );
                                            }}
                                        />
                                    )}
                                </tbody>
                            </HTMLTable>
                            <Box>
                                <Button onClick={() => boundArrayHelpers.push(emptyField)}
                                        disabled={create}>
                                            Добавить поле данных
                                </Button>
                            </Box>
                        </div>
                        <div className={Classes.DIALOG_FOOTER}>
                            <div className={Classes.DIALOG_FOOTER_ACTIONS}>
                                <Button
                                    onClick={ () => {
                                            var doDelete = window.confirm("Удалить целую категорию?");
                                            if (doDelete) { 
                                                rootStore.objectsStore.delCategory(category.ID_CAT); 
                                                onClose();
                                            }
                                        }
                                    }
                                    intent={Intent.DANGER}
                                    disabled={create}>
                                    Удалить 
                                </Button>

                                <Button
                                    onClick={handleSubmit}
                                    intent={Intent.PRIMARY}
                                    disabled={isSubmitting || !(isValid && dirty)}>
                                    {getButtonTitle()} 
                                </Button>
                            </div>
                        </div>
                    </>
                )}
            </Formik>
        </Dialog>
    );
});

export default CategoryEditDialog;
