import React, { useState, useEffect } from 'react';
import {
    Button,
    Tooltip,
    InputGroup,
    FormGroup,
    Intent,
    Callout,
    Card,
    Switch,
    ControlGroup,
    HTMLSelect,
    MenuItem,
    H2,
} from '@blueprintjs/core';
import { Select } from '@blueprintjs/select';
import { observer } from 'mobx-react-lite';import { useHistory, useLocation } from 'react-router-dom';
import { useStores } from '../hooks/useStores';
import { Helmet } from 'react-helmet-async';

const Login = observer(props => {
    let [showPassword, setShowPassword] = useState(false);
    let [username, setUsername] = useState('');
    let [password, setPassword] = useState('');
    let [customServer, setCustomServer] = useState(null);
    let [useCustomServer, setUseCustomServer] = useState(false);
    let [customServerOk, setCustomServerOk] = useState(null);
    let [customServerProtocol, setCustomServerProtocol] = useState('http');
    let [error, setError] = useState(null);
    let [hasLogins, setHasLogins] = useState(null);

    let [calloutText, setCalloutText] = useState(null);

    const { rootStore } = useStores();
    let { servers, getServers, addServer } = rootStore.serversStore;
    let { login, isLoggedIn, incorrectPass, server, apiUnAuthCustomRequest, changeServer } = rootStore.loginStore;
    const { serverStatus } = rootStore.infoStore;

    let history = useHistory();
    let location = useLocation();

    let { from } = location.state || { from: { pathname: '/' } };
    let { reason } = location.state || { undefined };

    let customServerInput = `${customServerProtocol}://${customServer}`.toLowerCase();

    const cardStyle = {
        width: 500,
        alignSelf: 'center',
        margin: 'auto',
    };

    let insecureServerError =
        process.env.REACT_APP_MULTI_API &&
        useCustomServer &&
        customServerProtocol === 'http' &&
        window.location.protocol === 'https:';

    useEffect(() => {
        if (isLoggedIn) {
            addServer(server, undefined, undefined);
            history.replace(from);
        }
        if (serverStatus === false) {
            setError('Сервер не отвечает');
        }
        if (incorrectPass && hasLogins) {
            setError('Неправильный логин или пароль');
        }
    });

    useEffect(() => {
        if (from.pathname !== '/') {
            if (location.state.reason) {
                setError(reason);
            } else {
                setError('Чтобы просматривать эту страницу, войдите на сайт');
            }
        }
        getServers();
    }, []);

    const lockButton = (
        <Tooltip content={`${showPassword ? 'Скрыть' : 'Показать'} пароль`}>
            <Button
                icon={showPassword ? 'unlock' : 'lock'}
                intent={Intent.WARNING}
                minimal={true}
                onClick={() => setShowPassword(!showPassword)}
            />
        </Tooltip>
    );

    let alert = text => {
        return (
            <FormGroup>
                <Callout intent={Intent.WARNING}>{text}</Callout>
            </FormGroup>
        );
    };

    let doLogin = () => {
        setHasLogins(true);
        if (useCustomServer) {
            apiUnAuthCustomRequest(
                null,
                'sys-status',
                customServerInput,
                res => {
                    changeServer(customServerInput);
                    login(username, password);
                },
                err => {
                    setError(err);
                    handleCustomServerState(false);
                }
            );
        } else {
            login(username, password);
        }
    };

    function handleSwitch() {
        setUseCustomServer(!useCustomServer);
        handleCustomServerState();
    }

    let options = ['HTTP', 'HTTPS'];
    let customServerForm = (
        <>
            <FormGroup>
                <Switch label="Другой сервер" defaultChecked={useCustomServer} onChange={handleSwitch} />
                <ControlGroup>
                    <HTMLSelect
                        options={options}
                        disabled={!useCustomServer}
                        value={customServerProtocol.toUpperCase()}
                        onChange={e => {
                            setCustomServerProtocol(e.target.value.toLowerCase());
                            handleCustomServerState(null);
                        }}
                    />
                    <InputGroup
                        disabled={!useCustomServer}
                        placeholder="server:port"
                        value={customServer}
                        onChange={e => {
                            setCustomServer(e.target.value);
                            handleCustomServerState(null);
                        }}
                    />
                    <Button
                        text="Тест"
                        disabled={!useCustomServer || (!!!customServer && !insecureServerError)}
                        onClick={() => {
                            customServerTest();
                            handleCustomServerState(null);
                        }}
                    />
                </ControlGroup>
            </FormGroup>
            <FormGroup>
                <Select
                    items={servers}
                    filterable={false}
                    disabled={!useCustomServer}
                    noResults={<MenuItem disabled={true} text="Нет сохраненных адресов" />}
                    itemRenderer={itemRenderer}
                    onItemSelect={handleClick}>
                    <Button text={'Сохраненные адреса'} rightIcon="caret-down" />
                </Select>
            </FormGroup>
        </>
    );

    function itemRenderer(item, { handleClick }) {
        return <MenuItem key={item.server} text={item.server} onClick={handleClick} shouldDismissPopover={true} />;
    }

    function handleClick(item) {
        var url = new URL(item.server);
        setCustomServerProtocol(url.protocol.replace(':', '').toUpperCase());
        setCustomServer(url.host);
        setCustomServerOk(null);
    }

    let title = (
        <span>
            Сервер <a href={customServerInput}>{customServerInput}</a> недоступен
        </span>
    );
    let serverFailed = (
        <FormGroup>
            <Callout intent={Intent.WARNING} title={title}>
                {calloutText}
            </Callout>
        </FormGroup>
    );

    let serverOk = (
        <FormGroup>
            <Callout intent={Intent.SUCCESS}>{calloutText}</Callout>
        </FormGroup>
    );

    let insecureServer = (
        <FormGroup>
            <Callout intent={Intent.DANGER}>Доступ к серверу по HTTP отсюда невозможен</Callout>
        </FormGroup>
    );

    function customServerTest() {
        apiUnAuthCustomRequest(
            null,
            'sys-status',
            customServerInput,
            res => {
                handleResponse(res);
            },
            err => {
                handleError(err);
            }
        );
    }

    function handleResponse(response) {
        setCalloutText(response.ServerNameFull);
        handleCustomServerState(true);
    }

    function handleError(err) {
        setCalloutText(err);
        handleCustomServerState(false);
    }

    function handleCustomServerState(state) {
        setCustomServerOk(state);
    }

    function loginButtonState() {
        if (useCustomServer) {
            return !(!!username && !!password && !!customServer);
        } else {
            return !(!!username && !!password) || !serverStatus;
        }
    }

    function loginButtonText() {
        if (useCustomServer && server !== customServerInput) {
            return 'Сменить сервер и войти';
        } else {
            return 'Войти';
        }
    }

    return (
        <>
            <Helmet>
                <title>Dew -- Логин</title>
            </Helmet>
            <Card style={cardStyle}>
                <H2>Вход в систему</H2>
                {!!error && alert(error)}
                <FormGroup label="Логин">
                    <InputGroup onChange={e => setUsername(e.target.value)} />
                </FormGroup>
                <FormGroup label="Пароль">
                    <InputGroup
                        type={showPassword ? 'text' : 'password'}
                        rightElement={lockButton}
                        onChange={e => setPassword(e.target.value)}
                    />
                </FormGroup>
                {process.env.REACT_APP_MULTI_API === "true" && props.customServer && customServerForm}
                {insecureServerError && insecureServer}
                {useCustomServer && customServerOk && serverOk}
                {useCustomServer && customServerOk === false && serverFailed}
                <FormGroup>
                    <Button text={loginButtonText()} intent="primary" onClick={doLogin} disabled={loginButtonState()} />
                </FormGroup>
            </Card>
        </>
    );
});

export default Login;
