import React, { useEffect, useState } from 'react';
import {
    Spinner,
    HTMLTable,
    ControlGroup,
    InputGroup,
    Button,
    FormGroup,
    NonIdealState,
    Callout,
    Intent,
    Classes,
    Dialog,
    NumericInput,
} from '@blueprintjs/core';
import { observer } from 'mobx-react-lite';import { Link, useHistory, useParams } from 'react-router-dom';
import { useStores } from '../hooks/useStores';
import { dateTime } from '../utils/dateTime';
import SelectCategories from './SelectCategories';
import PaginationBoxView from './pagination';

const Objects = observer(props => {
    let { catID } = useParams();

    const { rootStore } = useStores();
    let {
        objects,
        getObjects,
        getTypeByID,
        types,
        fetching,
        categories,
        searchMask,
        setSearchMask,
        setSelectedCategories,
        selectedCategories,
        count,
    } = rootStore.objectsStore;

    const { objectsPerPage, setObjectsPerPage } = rootStore.localStore;

    let history = useHistory();

    let [showSettings, setShowSettings] = useState(false);

    useEffect(() => {
        setSelectedCategories(catID);
    }, [catID, setSelectedCategories, categories]);

    useEffect(() => {
        if (searchMask === '') {
            getObjects(searchMask, true);
            return;
        }
        const delayDebounceFn = setTimeout(() => {
            getObjects(searchMask, true);
        }, 300);

        return () => clearTimeout(delayDebounceFn);
    }, [searchMask, getObjects, selectedCategories.length]);

    const handlePageClick = data => {
        let selected = data.selected;
        let offset = Math.ceil(selected * 3);

        getObjects(searchMask, false, offset);
    };

    const typeImage = id => {
        if (types.length) {
            let imgString = getTypeByID(id).IMG_NORMAL;
            return <img src={`data:image/png;base64,${imgString}`} alt="" />;
        }
    };

    const settingsDialog = (
        <Dialog
            icon="cog"
            isOpen={showSettings}
            onClose={() => {
                setShowSettings(false);
            }}
            title="Поиск объектов">
            <div className={Classes.DIALOG_BODY}>
                <FormGroup label="Объектов на странице" labelFor="text-input">
                    <NumericInput
                        placeholder="Введите число"
                        value={objectsPerPage}
                        onValueChange={number => setObjectsPerPage(number)}
                    />
                </FormGroup>
            </div>
            <div className={Classes.DIALOG_FOOTER}>
                <div className={Classes.DIALOG_FOOTER_ACTIONS}>
                    <Button onClick={() => setShowSettings(false)} intent={Intent.PRIMARY}>
                        Закрыть
                    </Button>
                </div>
            </div>
        </Dialog>
    );

    const objsList = (
        <>
            <HTMLTable interactive={true}>
                <thead>
                    <tr>
                        <th></th>
                        <th>ID</th>
                        <th>ID_CAT</th>
                        <th>ID_TYPE</th>
                        <th>Обозначение</th>
                        <th>Наименование</th>
                        <th>Дата создания</th>
                        <th>Дата изменения</th>
                    </tr>
                </thead>
                <tbody>
                    {objects.map(o => {
                        return (
                            <tr onClick={e => history.push(`/objects/${o.ID_CAT}/${o.ID}`)}>
                                <td>{typeImage(o.ID_TYPE)}</td>
                                <td>{o.ID}</td>
                                <td>{o.ID_CAT}</td>
                                <td>{o.ID_TYPE}</td>
                                <td>
                                    <code>
                                        <Link to={`/objects/${o.ID_CAT}/${o.ID}`}>{o.SPEC}</Link>
                                    </code>
                                </td>
                                <td>{o.NAME}</td>
                                <td style={{ minWidth: 150 }}>{dateTime(o.DateOfCreation)}</td>
                                <td style={{ minWidth: 150 }}>{dateTime(o.DateOfModification)}</td>
                            </tr>
                        );
                    })}
                </tbody>
            </HTMLTable>
        </>
    );

    let clearButton = <Button icon="cross" disabled={!searchMask} minimal={true} onClick={() => setSearchMask('')} />;

    let noResults = <NonIdealState icon={'search'} title={`Нет результатов по запросу «${searchMask}»`} />;

    let paginator = (
        <>
            <PaginationBoxView
                previousLabel={'<'}
                nextLabel={'>'}
                breakLabel={'...'}
                pageCount={Math.floor(count / objectsPerPage)}
                marginPagesDisplayed={2}
                pageRangeDisplayed={5}
                onPageChange={handlePageClick}
                containerClassName={'bp3-button-group'}
                activeLinkClassName={'bp3-active bp3-intent-primary'}
                pageLinkClassName={'bp3-button'}
                breakLinkClassName={'bp3-button'}
                previousLinkClassName={'bp3-button'}
                nextLinkClassName={'bp3-button'}
                disabledClassName={'bp3-disabled'}
            />
        </>
    );

    function highlightText(text, query) {
        let lastIndex = 0;
        const words = query
            .split(/\s+/)
            .filter(word => word.length > 0)
            .map(escapeRegExpChars);
        if (words.length === 0) {
            return [text];
        }
        const regexp = new RegExp(words.join('|'), 'gi');
        const tokens = [];
        while (true) {
            const match = regexp.exec(text);
            if (!match) {
                break;
            }
            const length = match[0].length;
            const before = text.slice(lastIndex, regexp.lastIndex - length);
            if (before.length > 0) {
                tokens.push(before);
            }
            lastIndex = regexp.lastIndex;
            tokens.push(<strong key={lastIndex}>{match[0]}</strong>);
        }
        const rest = text.slice(lastIndex);
        if (rest.length > 0) {
            tokens.push(rest);
        }
        return tokens;
    }

    function escapeRegExpChars(text) {
        return text.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, '\\$1');
    }

    const noCat = !!catID && !!categories.length && !categories.some(c => c.ID_CAT === Number(catID));

    return (
        <>
            {settingsDialog}

            <ul className="bp3-breadcrumbs">
                <li>
                    <span className="bp3-breadcrumb bp3-breadcrumb-current">Объекты</span>
                </li>
            </ul>
            <FormGroup label={'Фильтр'}>
                <ControlGroup>
                    <InputGroup
                        style={{ width: 500 }}
                        value={searchMask}
                        placeholder="Обозначение или наименование"
                        rightElement={clearButton}
                        onChange={e => setSearchMask(e.target.value)}
                    />
                    <Button icon="cog" onClick={() => setShowSettings(true)}>
                        Настройки поиска
                    </Button>
                    {fetching && (
                        <div style={{ marginLeft: 10, display: 'flex', alignItems: 'center' }}>
                            <Spinner size={Spinner.SIZE_SMALL} />
                        </div>
                    )}
                </ControlGroup>
            </FormGroup>
            <SelectCategories
                callback={() => {
                    getObjects(searchMask, true);
                }}
            />
            {noCat && (
                <FormGroup>
                    <Callout intent={Intent.DANGER}>Категория {catID} не найдена!</Callout>
                </FormGroup>
            )}
            <div style={{ height: '100%' }}>
                {!fetching && (
                    <div style={{ height: '100%' }}>
                        {objects.length !== 0 && objsList}
                        {objects.length === 0 && noResults}
                    </div>
                )}
                {objects.length === 0 && fetching && (
                    <div style={{ height: '100%', display: 'flex', justifyContent: 'center' }}>
                        <Spinner />
                    </div>
                )}
            </div>
            {(objects.length || fetching) && (
                <div style={{ display: 'flex', flexDirection: 'row' }}>
                    <div style={{ alignSelf: 'center' }}>
                        <p>
                            {selectedCategories.length === categories.length ? 'В базе: ' : 'По фильтру: '} {count}
                        </p>
                    </div>
                    <div style={{ flexGrow: 1, textAlign: 'center' }}>{count > objectsPerPage && paginator}</div>
                    <div style={{ alignSelf: 'center' }}>
                        <Button>Загрузить все</Button>
                    </div>
                </div>
            )}
        </>
    );
});

export default Objects;
