import { Button, Checkbox, FormGroup, HTMLTable, InputGroup, Intent } from '@blueprintjs/core';
import { Formik } from 'formik';
import { observer } from 'mobx-react-lite';
import React, { useEffect } from 'react';
import styled from 'styled-components';
import * as yup from 'yup';
import { useStores } from '../../hooks/useStores';

const ServerSettings = observer(({ className }) => {
    const { rootStore } = useStores();
    const { instanceSettings, getInstanceSettings, setInstanceSettings } = rootStore.infoStore;

    useEffect(() => {
        getInstanceSettings();
    }, [getInstanceSettings]);

    const validationSchema = yup.object({
        serverAddress: yup.string().required(),
        serverPort: yup.string().required(),
        serverUpdateClientFolderName: yup.string().required(),
        serverAPIPrefix: yup.string().required(),
        serverNameShort: yup.string().required(),
        serverNameFull: yup.string().required(),
        dbType: yup.string().required(),
        dbConnectionString: yup.string().required(),
        dbSchema: yup.string().required(),
        usingCompression: yup.bool(),
        allowCors: yup.bool().required(),
        builtInAccount: yup.bool().required(),
    });

    return (
        <div className={className}>
            <Formik
                enableReinitialize
                initialValues={{
                    serverAddress: instanceSettings.ServerAddress,
                    serverPort: instanceSettings.ServerPort,
                    serverUpdateClientFolderName: instanceSettings.ServerUpdateClientFolderName,
                    serverAPIPrefix: instanceSettings.ServerAPIPrefix,
                    serverNameShort: instanceSettings.ServerNameShort,
                    serverNameFull: instanceSettings.ServerNameFull,
                    dbType: instanceSettings.DBType,
                    dbConnectionString: instanceSettings.DBConnectionString,
                    dbSchema: instanceSettings.DBSchema,
                    usingCompression: instanceSettings.UsingCompression,
                    allowCors: instanceSettings.AllowCors,
                    builtInAccount: instanceSettings.BuiltInAccount,
                }}
                validationSchema={validationSchema}
                onSubmit={(values, actions, create) => {
                    setInstanceSettings(values);
                    actions.setSubmitting(false);
                }}>
                {({ handleSubmit, values, errors, setFieldValue, isSubmitting, isValid, dirty, touched }) => (
                    <>
                        <FormGroup label="Адрес сервера" inline>
                            <InputGroup
                                value={values.serverAddress}
                                onChange={e => setFieldValue('serverAddress', e.target.value)}
                                intent={errors.serverAddress && touched.serverAddress ? Intent.DANGER : Intent.NONE}
                            />
                        </FormGroup>
                        <FormGroup label="Порт" inline>
                            <InputGroup
                                value={values.serverPort}
                                onChange={e => setFieldValue('serverPort', e.target.value)}
                                intent={errors.serverPort && touched.serverPort ? Intent.DANGER : Intent.NONE}
                            />
                        </FormGroup>
                        <FormGroup label="Директория клиента" inline>
                            <InputGroup
                                value={values.serverUpdateClientFolderName}
                                onChange={e => setFieldValue('serverUpdateClientFolderName', e.target.value)}
                                intent={
                                    errors.serverUpdateClientFolderName && touched.serverUpdateClientFolderName
                                        ? Intent.DANGER
                                        : Intent.NONE
                                }
                            />
                        </FormGroup>
                        <FormGroup label="Маска поиска модулей" inline>
                            <InputGroup
                                value={values.serverAPIPrefix}
                                onChange={e => setFieldValue('serverAPIPrefix', e.target.value)}
                                intent={errors.serverAPIPrefix && touched.serverAPIPrefix ? Intent.DANGER : Intent.NONE}
                            />
                        </FormGroup>
                        <FormGroup label="Сокращенное имя сервера" inline>
                            <InputGroup
                                value={values.serverNameShort}
                                onChange={e => setFieldValue('serverNameShort', e.target.value)}
                                intent={errors.serverNameShort && touched.serverNameShort ? Intent.DANGER : Intent.NONE}
                            />
                        </FormGroup>
                        <FormGroup label="Полное имя сервера" inline>
                            <InputGroup
                                value={values.serverNameFull}
                                onChange={e => setFieldValue('serverNameFull', e.target.value)}
                                intent={errors.serverNameFull && touched.serverNameFull ? Intent.DANGER : Intent.NONE}
                            />
                        </FormGroup>
                        <FormGroup label="Тип БД" inline>
                            <InputGroup
                                value={values.dbType}
                                onChange={e => setFieldValue('dbType', e.target.value)}
                                intent={errors.dbType && touched.dbType ? Intent.DANGER : Intent.NONE}
                            />
                        </FormGroup>
                        <FormGroup label="Строка подключения к БД" inline>
                            <InputGroup
                                value={values.dbConnectionString}
                                onChange={e => setFieldValue('dbConnectionString', e.target.value)}
                                intent={
                                    errors.dbConnectionString && touched.dbConnectionString
                                        ? Intent.DANGER
                                        : Intent.NONE
                                }
                            />
                        </FormGroup>
                        <FormGroup label="Схема БД" inline>
                            <InputGroup
                                value={values.dbSchema}
                                onChange={e => setFieldValue('dbSchema', e.target.value)}
                                intent={errors.dbSchema && touched.dbSchema ? Intent.DANGER : Intent.NONE}
                            />
                        </FormGroup>
                        <FormGroup label="Сжатие" inline>
                            <Checkbox
                                checked={values.usingCompression}
                                onChange={e => setFieldValue('usingCompression', e.target.checked)}
                            />
                        </FormGroup>
                        <FormGroup label="CORS" inline>
                            <Checkbox
                                checked={values.allowCors}
                                onChange={e => setFieldValue('allowCors', e.target.checked)}
                            />
                        </FormGroup>
                        <FormGroup label="Встроенная учетная запись администратора" inline>
                            <Checkbox
                                checked={values.builtInAccount}
                                onChange={e => setFieldValue('builtInAccount', e.target.checked)}
                            />
                        </FormGroup>
                        <Button
                            onClick={handleSubmit}
                            intent={Intent.PRIMARY}
                            disabled={isSubmitting || !(isValid && dirty)}>
                            Сохранить
                        </Button>
                    </>
                )}
            </Formik>
        </div>
    );
});

const StyledServerSettings = styled(ServerSettings)`
    & .bp3-label {
        width: 300px;
    }
    & .bp3-form-content {
        width: 100%;
    }
`;

export default StyledServerSettings;
