import { extendObservable, makeAutoObservable } from 'mobx';

class LoginStore {
    username;
    password;
    session = undefined; // Если явно не объявить undefined, то не обсервится
    server = undefined;
    userInfo = {};
    incorrectPass;
    lastError;
    loading = false;
    apis;

    constructor(rootStore) {
        this.rootStore = rootStore;
        makeAutoObservable(this);

        this.server = process.env.REACT_APP_API;

        const server = localStorage.getItem('server');

        if (!!server && server !== process.env.REACT_APP_API) {
            this.server = server;
        }

        window.lll = this;
    }

    clearLastError = () => {
        this.lastError = undefined;
    }

    login = (username, password) => {
        this.username = username;
        this.password = password;

        const req = {
            UserName: this.username,
            PassWord: this.password,
        };

        this.apiUnAuthRequest(req, 'user-login', res => {
            if (res.ResultCode === 1) {
                this.session = res.SessionID;

                localStorage.setItem('username', this.username);
                localStorage.setItem('session', this.session);

                this.rootStore.infoStore.getServerInfo();

                return true;
            }
            if (res.ResultCode === 6001 || res.ResultCode === 6000) {
                this.incorrectPass = true;
                return true;
            }
            return false;
        });
    };

    sessionLogin = (session, callback) => {
        this.loading = true;

        if (session) {
            this.apiRequest(
                null,
                'user-info',
                session,
                res => {
                    if (res.ID > 0) {
                        this.userInfo = res;
                        this.username = res.U_NAME;
                        this.session = session;

                        localStorage.setItem('username', this.username);
                        localStorage.setItem('session', this.session);

                        this.loading = false;
                        return callback(true);
                    } else {
                        this.loading = false;
                        return callback(false);
                    }
                },
                () => {
                    this.loading = false;
                    return callback(false);
                }
            );
        } else {
            this.loading = false;
            return callback(false);
        }
    };

    logout = () => {
        this.username = undefined;
        this.password = undefined;
        this.session = undefined;
        this.incorrectPass = undefined;

        localStorage.removeItem('username');
        localStorage.removeItem('session');
    };

    logoutToDefault = () => {
        this.changeServerToDefault();
        this.rootStore.infoStore.reset();
        this.rootStore.infoStore.getServerInfo();
        this.logout();
    };

    sleep = time => {
        return new Promise(resolve => setTimeout(resolve, time));
    };

    tryLogin = callback => {
        this.loading = true;

        let session = localStorage.getItem('session');
        let username = localStorage.getItem('username');

        if (session && username) {
            this.apiRequest(
                null,
                'user-info',
                session,
                res => {
                    if (res.ID > 0) {
                        this.userInfo = res;
                        this.username = res.U_NAME;
                        this.session = session;
                        this.loading = false;
                        return callback(true);
                    } else {
                        this.loading = false;
                        return callback(false);
                    }
                },
                () => {
                    this.loading = false;
                    return callback(false);
                }
            );
        } else {
            this.loading = false;
            return callback(false);
        }
    };

    getUserInfo = () => {
        this.apiRequest(null, 'user-info', this.session, res => {
            this.userInfo = res;
        });
    };

    get isLoggedIn() {
        console.log('checking')
        return !!this.session;
    }

    hasPermission = permission => {
        if (!this.userInfo || !this.userInfo.PermissionNames) return false;
        return this.userInfo.PermissionNames.includes(permission);
    };

    get isAdmin() {
        if (!this.userInfo || !this.userInfo.PermissionNames) return false;
        return this.userInfo.PermissionNames.includes('canAdminUsers');
    }

    changeServerToDefault = () => {
        this.server = process.env.REACT_APP_API;
        localStorage.removeItem('server');
    };

    changeServer = server => {
        this.server = server;
        localStorage.setItem('server', server);
    };

    apiUnAuthRequest = (reqBody, apiName, callback, onError) => {
        return this.apiRequest(reqBody, apiName, null, callback, onError, null, this.server);
    };

    apiUnAuthCustomRequest = (reqBody, apiName, customServer, callback, onError, onDone) => {
        return this.apiRequest(reqBody, apiName, null, callback, onError, onDone, customServer);
    };

    apiAuthRequest = (reqBody, apiName, callback, onError, onDone, onProgress) => {
        return this.apiRequest(reqBody, apiName, this.session, callback, onError, onDone, this.server, onProgress);
    };

    apiRequest = (reqBody, apiName, session, callback, onError, onDone, server, onProgress) => {
        let self = this;

        let api = !!server ? server : this.server;
        var xhr = new XMLHttpRequest();

        try {
            xhr.open('POST', `${api}/${apiName}`);
        } catch (error) {
            if (process.env.NODE_ENV === 'development') {
                console.log(apiName, '←', error);
            }
            return;
        }

        xhr.setRequestHeader('Content-Type', 'application/json');
        if (session) {
            xhr.setRequestHeader('X-deep-session', session);
        }

        xhr.onreadystatechange = function (e) {
            if (xhr.readyState === 4) {
                if (xhr.status === 200) {
                    // WTF
                    try {
                        var res = JSON.parse(xhr.responseText);
                    } catch {
                        self.lastError = 'Сервер вернул "OK"';
                        if (onError && typeof onError === 'function') {
                            self.lastError = 'Сервер вернул "OK"';
                            onError('Сервер вернул "OK"');
                        }
                    }

                    if (process.env.NODE_ENV === 'development') {
                        console.log(apiName, '←', res);
                    }

                    if (xhr.responseText === 'OK') {
                        if (onError && typeof onError === 'function') {
                            self.lastError = 'Сервер вернул "OK';
                            onError('Сервер вернул "OK"');
                        }
                    } else {
                        callback(res);
                    }
                } else {
                    if (process.env.NODE_ENV === 'development') {
                        console.log(apiName, '←', res);
                    }

                    if (onError != null) {
                        if (xhr.status === 0) {
                            onError('Ошибка соединения');
                        } else {
                            onError(xhr.status + ': ' + xhr.statusText);
                        }
                    }
                }
                if (onDone && typeof onDone === 'function') onDone();
            }
        };

        xhr.onerror = () => {
            if (onError && typeof onError === 'function') onError('Ошибка соединения');
        };

        xhr.onprogress = e => {
            if (onProgress && typeof onProgress === 'function' && e.lengthComputable) {
                var percentComplete = (e.loaded / e.total) * 100;
                onProgress(percentComplete);
            }
        };

        if (process.env.NODE_ENV === 'development') {
            console.log(apiName, '→', reqBody);
        }
        xhr.send(JSON.stringify(reqBody));
    };

    sendFeedback = (text, pngUrl, callback) => {
        // эмуляция ответа сервера
        setTimeout(() => {
            callback();
        }, 1000);

        // should be sent...
        console.log(this.username);
        console.log(this.session);
        console.log(window.navigator.userAgent);
        console.log(window.location.href);
        console.log(pngUrl);
        console.log(text);
    };

    getAPIs = callback => {
        this.apiAuthRequest(null, 'api-reference', res => {
            this.apis = res;
            callback();
        });
    };

    getAPIRequest = (apiName, callback) => {
        const req = { ApiName: apiName };
        this.apiAuthRequest(req, 'api-reference-get-request', res => {
            if (!!this.apis) {
                let currentApi = null;
                let currentAsm = null;
                for (let asm of this.apis.Assemblies) {
                    for (let api of asm.Apis) {
                        if (api.ApiName === apiName) {
                            currentApi = api;
                            currentAsm = asm;
                            break;
                        }
                    }
                }
                extendObservable(currentApi, res);
                callback(currentApi, currentAsm);
            }
        });
    };
}

export default LoginStore;
