import { extendObservable, makeAutoObservable, reaction } from 'mobx';

class ObjectsStore {
    categories = [];
    objects = [];
    types = [];
    relations = [];
    fetching = false;
    currentObject = {};
    searchMask = '';
    selectedCategories = [];

    constructor(rootStore) {
        this.rootStore = rootStore;
        makeAutoObservable(this);

        const disposer = reaction(
            () => this.rootStore.loginStore.session,
            () => {
                this.getCategories();
                this.getTypes();
                this.getRelations();
            }
        );
    }

    getCategories = () => {
        const req = { GetAll: true };
        this.rootStore.loginStore.apiAuthRequest(req, 'api-categories-get', res => {
            this.categories = res.Categories;
        });
    };

    getCategoryColumns = catID => {
        const req = { GetAll: true, CategoryID: catID };
        this.rootStore.loginStore.apiAuthRequest(req, 'api-categories-dcol-get', res => {
            const cat = this.categories.find(x => x.ID_CAT === catID);
            if (cat.CategoryDataColumns) {
                cat.CategoryDataColumns = res.CategoryDataColumns;
            } else {
                extendObservable(cat, res);
            }
        });
    };

    checkTable = (tableName, resultMsg) => {
        const req = {
            TableName: tableName,
        };

        this.rootStore.loginStore.apiAuthRequest(req, 'api-db-checktable', res => {
            if (res.isExist) {
                //window.alert("Таблица существует!")
                resultMsg('Таблица существует!');
            } else {
                resultMsg('Таблица НЕ существует!');
            }
            this.fetching = false;
        });
    };

    setCategory = (c, values) => {
        if (values.create) {
            //...create category
            // "api-categories-add"
            const req = {
                newCategories: [
                    {
                        //ID_CAT: c.ID_CAT,
                        NAME: values.fullName,
                        NAME_SHORT: values.shortName,
                        DB_TBL_NAME: values.tableName,
                    },
                ],
            };
            this.rootStore.loginStore.apiAuthRequest(req, 'api-categories-add', res => {
                this.fetching = false;
                this.getCategories();
            });
        } else {
            const req = {
                CategoryNewData: {
                    ID_CAT: c.ID_CAT,
                    NAME: values.fullName,
                    NAME_SHORT: values.shortName,
                    DB_TBL_NAME: values.tableName,
                },
                doChangeCategoryData: true,
            };
            this.rootStore.loginStore.apiAuthRequest(req, 'api-categories-set', res => {
                
                let arrayToAdd = [];

                for (var i = 0; i < values.categoryDataColumns.length; i++) {
                    if (values.categoryDataColumns[i].ID_CAT_DCOL) {
                        const req_dcol_set = {
                            doChangeCategoryDataColumnData: true,
                            categoryID: values.categoryDataColumns[i].ID_CAT,
                            categoryDataColumnID: values.categoryDataColumns[i].ID_CAT,
                            CategoryDataColumnNewData: values.categoryDataColumns[i],
                        };
                        this.rootStore.loginStore.apiAuthRequest(req_dcol_set, 'api-categories-dcol-set', res => {
                            console.log('Изменили колонку: ' + res);
                        });
                    } else {
                        values.categoryDataColumns[i].ID_CAT = c.ID_CAT;
                        arrayToAdd.push(values.categoryDataColumns[i]);
                    }
                }

                if (arrayToAdd.length > 0) {
                    const req_dcol_add = {
                        CategoryID: c.ID_CAT,
                        newCategoryDataColumns: arrayToAdd,
                    };
                    this.rootStore.loginStore.apiAuthRequest(req_dcol_add, 'api-categories-dcol-add', res => {
                        console.log('Добавили колонок: ' + res);
                    });
                }

                this.fetching = false;
                this.getCategories();
            });
        }
    };

    delCategory = id => {
        const req = {
            deleteCategoriesIDs: [id],
        };
        this.rootStore.loginStore.apiAuthRequest(req, 'api-categories-delete', res => {
            this.fetching = false;
            this.getCategories();
        });
    };

    delCategoryColumn = (catID, colID) => {
        const req = {
            CategoryID: catID,
            deleteCategoryDataColumnsIDs: [colID]
        };
        this.rootStore.loginStore.apiAuthRequest(req, 'api-categories-dcol-delete', res => {
            this.fetching = false;
            this.getCategories();
        });
    };

    getRelations = () => {
        const req = { GetAll: true };
        this.rootStore.loginStore.apiAuthRequest(req, 'api-relation-types-get', res => {
            this.relations = res.RelationTypes;
        });
    };

    setSelectedCategories = catID => {
        if (this.categories.length === 0) {
            return;
        }

        if (catID) {
            let cat = this.categories.find(c => String(c.ID_CAT) === catID);

            if (!!cat) {
                this.selectedCategories.clear();
                this.selectedCategories.push(cat);
            } else {
                this.selectedCategories = [...this.categories];
            }
        } else {
            // Запрос для поиска может быть выполнен и без категорий
            // this.selectedCategories = [...this.categories];
        }
    };

    get getFilteredCategories() {
        return this.categories;
    }

    getTypes = () => {
        const req = { GetAll: true };
        this.rootStore.loginStore.apiAuthRequest(req, 'api-types-get', res => {
            this.types = res.ObjectTypes;
        });
    };

    getAllObjects = () => {
        const req = { GetAll: true, CategoryID: 10001 };
        this.fetching = true;
        this.rootStore.loginStore.apiAuthRequest(req, 'api-objects-get', res => {
            this.objects = res.Objects;
            this.fetching = false;
        });
    };

    getAllObjectsByCatID = catID => {
        const req = { GetAll: true, CategoryID: catID };
        this.fetching = true;
        this.rootStore.loginStore.apiAuthRequest(req, 'api-objects-get', res => {
            res.Objects.forEach(o => {
                const req = { CategoryID: o.ID_CAT, ObjectID: o.ID };
                this.rootStore.loginStore.apiAuthRequest(req, 'api-objects-getpars', res => {
                    o.Pars = res.Pars;
                    this.objects.push(o);
                });
                this.fetching = false;
            });
        });
    };

    count = 0;

    getObjects = (searchMask, newSearch = false, offset = 0) => {
        let batch = this.rootStore.localStore.objectsPerPage;

        this.objects.clear();

        if (newSearch) {
            this.count = 0;
        }

        // Запрос для поиска может быть выполнен и без категорий,
        // это исключает "гонку", когда задается полный список категорий, при первом открытии
        //
        // if (this.selectedCategories.length === 0) {
        //     this.count = 0;
        //     return;
        // }

        this.fetching = true;
        const req = { SearchMask: `%${searchMask}%`, CategoryIDs: this.selectedCategories.map(c => c.ID_CAT) };
        this.rootStore.loginStore.apiAuthRequest(req, 'api-objects-search', res => {
            this.count = res.FoundObjectKeys.length;

            if (res.FoundObjectKeys.length === 0) {
                this.fetching = false;
            } else {
                let arr = res.FoundObjectKeys.slice(offset, offset + batch);

                arr.map(k => {
                    const req = { GetAll: false, ObjectIDs: [k.ID], CategoryID: k.ID_CAT };
                    this.rootStore.loginStore.apiAuthRequest(req, 'api-objects-get', resObj => {
                        this.objects.push(resObj.Objects[0]);
                        if (k === arr[arr.length - 1]) {
                            this.fetching = false;
                        }
                    });
                });
            }
        });
    };

    getObject = (id, idCat) => {
        this.fetching = true;
        const req = { GetAll: false, ObjectIDs: [id], CategoryID: idCat };
        this.rootStore.loginStore.apiAuthRequest(req, 'api-objects-get', resObj => {
            this.currentObject = resObj.Objects[0];
            this.fetching = false;
        });
    };

    getPars = (id, idCat) => {
        const req = { CategoryID: idCat, ObjectID: id };
        this.rootStore.loginStore.apiAuthRequest(req, 'api-objects-getpars', res => {
            extendObservable(this.currentObject, res);
        });
    };

    groupBy(xs, f) {
        return xs.reduce((r, v, i, a, k = f(v)) => ((r[k] || (r[k] = [])).push(v), r), {});
    }

    getObjectRelations = (id, idCat) => {
        // I have no idea how and why it works
        // Total hours wasted here: 4
        this.fetching = true;
        const req = { GetAll: true, GetAll_id_object: id, GetAll_id_cat: idCat };
        this.rootStore.loginStore.apiAuthRequest(req, 'api-relations-get', res => {
            let rels = this.groupBy(res.Relations, c => c.ID_REL_TYPE);
            let relsObjects = [];

            for (const [key, value] of Object.entries(rels)) {
                const relInfo = this.relations.find(r => r.ID_REL_TYPE === Number(key));

                let name;
                if (!!relInfo) {
                    name = relInfo.NAME;
                } else {
                    name = `Не найдена информация об отношении ${key}`;
                }

                relsObjects.push({
                    id: key,
                    name: name,
                    objects: value,
                });
            }

            for (let rels of relsObjects) {
                for (let obj of rels.objects) {
                    const req = { GetAll: false, ObjectIDs: [obj.REL_ID_OBJECT], CategoryID: obj.REL_ID_CAT };
                    this.rootStore.loginStore.apiAuthRequest(req, 'api-objects-get', resObj => {
                        obj.object = resObj.Objects[0];

                        if (
                            relsObjects[relsObjects.length - 1] === rels &&
                            rels.objects[rels.objects.length - 1] === obj
                        ) {
                            let obj = {
                                relsArr: relsObjects,
                            };
                            extendObservable(this.currentObject, obj);
                            this.fetching = false;
                        }
                    });
                }
            }
        });
    };

    getTypeByID = id => {
        return this.types.filter(t => t.ID_TYPE === id)[0];
    };

    setSearchMask = input => {
        this.searchMask = input;
    };
}

export default ObjectsStore;
